/**
 * Copyright (c) 2002, Vienna University of Technology, Real-Time Systems Group.
 * All rights reserved.
 *
 * Contact: sem182@vmars.tuwien.ac.at
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 **/

#ifndef I_msem182_h
#define I_msem182_h

/* SCCSId:	"@(#)	msem182.h	2.3	6/27/94	hp";
 *
 * Purpose: 	Convenience functions for Semaphores
 *
 * Motivation:	The semop function is hard to use and provides
 *              functionality which is seldom used or even annoying
 *		(like the fact that 0 isn't a nop). These functions
 *		should provide a cleaner interface.
 *
 * Modification history:
 *	2.1	Changed the semantics of mP and mV. The arguments now
 *		specify the index of the semaphore to operate on, 
 *		not the operation.
 *	1.1	got the version from the solution to Ex. 2.11 
 *		(which was derived from an out-of-date version of 
 *		sem182.c which had been heavily hacked by gusti) and
 *		adapted it to the STBS and the newest version of
 *		sem182.c.
 */
#include <sys/types.h>

/*** function mseminit
 * 
 * Purpose: Create and initialize a semaphore field.
 *	    The SF must not exist yet.
 * In:      key Key of semaphore field
 *          nsems   Number of semaphores the field shall contain
 *          perm    permission bits for ugo
 *	    ...	    nsems initial values for semaphores
 * Returns: Semaphore descriptor or -1 in case of error.
 */

int mseminit( key_t key, int perm, int nsems, ...);


/*** function msemgrab
 *
 * Purpose:	grab existing semaphore field
 * In:		key	Key of SF.
 *		nsems	number of semaphores in the SF
 * Returns:	semid or -1 in case of error
 */
int msemgrab (key_t key, int nsems);


/*** function msemrm
 * 
 * Purpose: remove semaphore field
 * In:	    semid	id of SF to remove
 */
int msemrm(int semid);


/*** function mV
 *
 * Purpose: Dijkstra's V for a whole semaphore field.
 * In:	    semid	Id of SF
 *	    nsems	Number of Semaphores in the field.
 *			(I am not sure if I should check this. 
 *			It may help catch some errors, but too many
 *			are caught by the OS, and a smaller number 
 *			might be handy if you want only a work on a
 *			subset).
 *	    ...		nsems values indicating the semaphores on which
 *			a V operation should be performed.
 * Returns: -1 in case of error, else 0.
 */
int mV(int semid, int nsems, ...);


/*** function mP
 *
 * Purpose: Dijkstra's P for a whole semaphore field.
 * In:	    semid	Id of SF
 *	    nsems	Number of Semaphores in the field.
 *			(I am not sure if I should check this. 
 *			It may help catch some errors, but too many
 *			are caught by the OS, and a smaller number 
 *			might be handy if you want only a work on a
 *			subset).
 *	    ...		nsems values indicating the semaphores on which
 *			a V operation should be performed.
 * Returns: -1 in case of error, else 0.
 */
int mP(int semid, int nsems, ...);

#endif /* I_msem182_h */
